/** @file selector.c
 *  @brief This module contains continuous tasks for cava (spectrum analyzer) comunication
 */

/***************************************************************************************************
 * Includes
 **************************************************************************************************/
#include "selector.h"

#include <errno.h>
#include <pthread.h>
#include <string.h>
#include <unistd.h>

#include "../../drivers/selector/rpi_selector.h"
#include "../../rpi_param.h"

#include "log.h"

/***************************************************************************************************
 * Preprocessor Constants and Macros
 **************************************************************************************************/

#define LISTENER_INTERVAL 500000

/***************************************************************************************************
 * Type and Contant Definitions
 **************************************************************************************************/

/***************************************************************************************************
 * Persistent Variables
 **************************************************************************************************/

pthread_t selectorListener;
/**
 * Boolean for interrupting the main thread loop
 */
bool isSelectorListenerRunning = false;

/***************************************************************************************************
 * Internal Function Prototypes
 **************************************************************************************************/

/**
 * @brief This function is used to thread main execution function
 *
 * @param   arg not used.
 *
 * @return  NULL.
 */
static void *listen_to_selector(void *arg);

/***************************************************************************************************
 * External Function Definitions
 **************************************************************************************************/

void selector_start() {
  isSelectorListenerRunning = true;
  pthread_create(&selectorListener, NULL, listen_to_selector, NULL);
}

void selector_stop() {
  isSelectorListenerRunning = false;
  if (pthread_join(selectorListener, NULL) != 0) {
    log_error("pthread_join: %s", strerror(errno));
  }
}

/***************************************************************************************************
 * Internal Function Definitions
 **************************************************************************************************/

static void *listen_to_selector(void *arg) {
  int previousPosition;
  selector_setup();

  while (isSelectorListenerRunning) {
    int newPosition = selector_get_position();
    if (newPosition != -1 && newPosition != previousPosition) {
      set_mode(newPosition);
      previousPosition = newPosition;
    }
    usleep(LISTENER_INTERVAL);
  }

  return NULL;
}